@@ -1,30 +1,13 @@
-# -*- coding: utf8 -*-
+# -*- coding: utf-8 -*-
 """GNUmed clinical patient record.
 
-This is a clinical record object intended to let a useful
-client-side API crystallize from actual use in true XP fashion.
-
-Make sure to call set_func_ask_user() and set_encounter_ttl()
-early on in your code (before cClinicalRecord.__init__() is
-called for the first time).
+Make sure to call set_func_ask_user() and set_encounter_ttl() early on in
+your code (before cClinicalRecord.__init__() is called for the first time).
 """
 #============================================================
 __author__ = "K.Hilbert <Karsten.Hilbert@gmx.net>"
 __license__ = "GPL v2 or later"
 
-#===================================================
-# TODO
-# Basically we'll probably have to:
-#
-# a) serialize access to re-getting data from the cache so
-#   that later-but-concurrent cache accesses spin until
-#   the first one completes the refetch from the database
-#
-# b) serialize access to the cache per-se such that cache
-#    flushes vs. cache regets happen atomically (where
-#    flushes would abort/restart current regets)
-#===================================================
-
 # standard libs
 import sys
 import logging
@@ -58,7 +41,6 @@ from Gnumed.business.gmDemographicRecord import get_occupations
 
 _log = logging.getLogger('gm.emr')
 
-_me = None
 _here = None
 #============================================================
 # helper functions
@@ -78,8 +60,6 @@ def set_func_ask_user(a_func = None):
 #============================================================
 class cClinicalRecord(object):
 
-	_clin_root_item_children_union_query = None
-
 	def __init__(self, aPKey=None, allow_user_interaction=True):
 		"""Fails if
 
@@ -88,46 +68,30 @@ class cClinicalRecord(object):
 		"""
 		self.pk_patient = aPKey			# == identity.pk == primary key
 
+		# FIXME: delegate to worker thread
 		# log access to patient record (HIPAA, for example)
 		cmd = u'SELECT gm.log_access2emr(%(todo)s)'
 		args = {'todo': u'patient [%s]' % aPKey}
 		gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}])
 
-		from Gnumed.business import gmSurgery, gmStaff
-		global _me
-		if _me is None:
-			_me = gmStaff.gmCurrentProvider()
+		from Gnumed.business import gmPraxis
 		global _here
 		if _here is None:
-			_here = gmSurgery.gmCurrentPractice()
-
-		# ...........................................
-		# this is a hack to speed up get_encounters()
-		clin_root_item_children = gmPG2.get_child_tables('clin', 'clin_root_item')
-		if cClinicalRecord._clin_root_item_children_union_query is None:
-			union_phrase = u"""
-SELECT fk_encounter from
-	%s.%s cn
-		inner join
-	(SELECT pk FROM clin.episode ep WHERE ep.fk_health_issue in %%s) as epi
-		on (cn.fk_episode =  epi.pk)
-"""
-			cClinicalRecord._clin_root_item_children_union_query = u'union\n'.join (
-				[ union_phrase % (child[0], child[1]) for child in clin_root_item_children ]
-			)
-		# ...........................................
-
-		self.__db_cache = {}
+			_here = gmPraxis.gmCurrentPraxisBranch()
 
 		# load current or create new encounter
 		if _func_ask_user is None:
 			_log.error('[_func_ask_user] is None')
 			print "*** GNUmed [%s]: _func_ask_user is not set ***" % self.__class__.__name__
+
+		# FIXME: delegate to worker thread ?
 		self.remove_empty_encounters()
+
 		self.__encounter = None
 		if not self.__initiate_active_encounter(allow_user_interaction = allow_user_interaction):
 			raise gmExceptions.ConstructorError, "cannot activate an encounter for patient [%s]" % aPKey
 
+		# FIXME: delegate to worker thread
 		gmAllergy.ensure_has_allergy_state(encounter = self.current_encounter['pk_encounter'])
 
 		# register backend notification interests
@@ -148,7 +112,7 @@ SELECT fk_encounter from
 	# messaging
 	#--------------------------------------------------------
 	def _register_interests(self):
-		gmDispatcher.connect(signal = u'encounter_mod_db', receiver = self.db_callback_encounter_mod_db)
+		gmDispatcher.connect(signal = u'clin.encounter_mod_db', receiver = self.db_callback_encounter_mod_db)
 
 		return True
 	#--------------------------------------------------------
@@ -175,7 +139,7 @@ SELECT fk_encounter from
 			raise ValueError('unsaved changes in active encounter, cannot switch to another one')
 
 		if self.current_encounter.same_payload(another_object = curr_enc_in_db):
-			_log.debug('encounter_mod_db received but no change to active encounter payload')
+			_log.debug('clin.encounter_mod_db received but no change to active encounter payload')
 			return True
 
 		# there was a change in the database from elsewhere,
@@ -188,26 +152,6 @@ SELECT fk_encounter from
 
 		return True
 	#--------------------------------------------------------
-	def db_callback_vaccs_modified(self, **kwds):
-		return True
-	#--------------------------------------------------------
-	def _health_issues_modified(self):
-		try:
-			del self.__db_cache['health issues']
-		except KeyError:
-			pass
-		return 1
-	#--------------------------------------------------------
-	def _db_callback_episodes_modified(self):
-#		try:
-#			del self.__db_cache['episodes']
-#		except KeyError:
-#			pass
-		return 1
-	#--------------------------------------------------------
-	def _clin_item_modified(self):
-		_log.debug('DB: clin_root_item modification')
-	#--------------------------------------------------------
 	# API: family history
 	#--------------------------------------------------------
 	def get_family_history(self, episodes=None, issues=None, encounters=None):
@@ -278,10 +222,11 @@ SELECT fk_encounter from
 	def get_latest_hospital_stay(self):
 		return gmEMRStructItems.get_latest_patient_hospital_stay(patient = self.pk_patient)
 	#--------------------------------------------------------
-	def add_hospital_stay(self, episode=None):
+	def add_hospital_stay(self, episode=None, fk_org_unit=None):
 		return gmEMRStructItems.create_hospital_stay (
 			encounter = self.current_encounter['pk_encounter'],
-			episode = episode
+			episode = episode,
+			fk_org_unit = fk_org_unit
 		)
 	#--------------------------------------------------------
 	def get_hospital_stay_stats_by_hospital(self, cover_period=None):
@@ -292,7 +237,7 @@ SELECT fk_encounter from
 
 		cmd = u"""
 			SELECT hospital, count(1) AS frequency
-			FROM clin.v_pat_hospital_stays
+			FROM clin.v_hospital_stays
 			WHERE
 				%s
 			GROUP BY hospital
@@ -325,10 +270,12 @@ SELECT fk_encounter from
 		if note.strip() == '':
 			_log.info('will not create empty clinical note')
 			return None
+		if isinstance(episode, gmEMRStructItems.cEpisode):
+			episode = episode['pk_episode']
 		status, data = gmClinNarrative.create_clin_narrative (
 			narrative = note,
 			soap_cat = soap_cat,
-			episode_id = episode['pk_episode'],
+			episode_id = episode,
 			encounter_id = self.current_encounter['pk_encounter']
 		)
 		if not status:
@@ -368,7 +315,7 @@ SELECT fk_encounter from
 			args['encs'] = tuple(encounters)
 
 		if soap_cats is not None:
-			where_parts.append(u'soap_cat IN %(cats)s')
+			where_parts.append(u'c_vn.soap_cat IN %(cats)s')
 			soap_cats = list(soap_cats)
 			args['cats'] = [ cat.lower() for cat in soap_cats if cat is not None ]
 			if None in soap_cats:
@@ -377,9 +324,11 @@ SELECT fk_encounter from
 
 		cmd = u"""
 			SELECT
-				c_vpn.*,
-				(SELECT rank FROM clin.soap_cat_ranks WHERE soap_cat = c_vpn.soap_cat) AS soap_rank
-			FROM clin.v_pat_narrative c_vpn
+				c_vn.*,
+				c_scr.rank AS soap_rank
+			FROM
+				clin.v_narrative c_vn
+					LEFT JOIN clin.soap_cat_ranks c_scr on c_vn.soap_cat = c_scr.soap_cat
 			WHERE %s
 			ORDER BY date, soap_rank
 		""" % u' AND '.join(where_parts)
@@ -395,7 +344,7 @@ SELECT fk_encounter from
 			filtered_narrative = filter(lambda narr: narr['date'] < until, filtered_narrative)
 
 		if providers is not None:
-			filtered_narrative = filter(lambda narr: narr['provider'] in providers, filtered_narrative)
+			filtered_narrative = filter(lambda narr: narr['modified_by'] in providers, filtered_narrative)
 
 		return filtered_narrative
 	#--------------------------------------------------------
@@ -444,147 +393,7 @@ order by
 		])
 		return rows
 	#--------------------------------------------------------
-	def get_text_dump_old(self):
-		# don't know how to invalidate this by means of
-		# a notify without catching notifies from *all*
-		# child tables, the best solution would be if
-		# inserts in child tables would also fire triggers
-		# of ancestor tables, but oh well,
-		# until then the text dump will not be cached ...
-		try:
-			return self.__db_cache['text dump old']
-		except KeyError:
-			pass
-		# not cached so go get it
-		fields = [
-			"to_char(modified_when, 'YYYY-MM-DD @ HH24:MI') as modified_when",
-			'modified_by',
-			'clin_when',
-			"case is_modified when false then '%s' else '%s' end as modified_string" % (_('original entry'), _('modified entry')),
-			'pk_item',
-			'pk_encounter',
-			'pk_episode',
-			'pk_health_issue',
-			'src_table'
-		]
-		cmd = "SELECT %s FROM clin.v_pat_items WHERE pk_patient=%%s order by src_table, clin_when" % ', '.join(fields)
-		ro_conn = self._conn_pool.GetConnection('historica')
-		curs = ro_conn.cursor()
-		if not gmPG2.run_query(curs, None, cmd, self.pk_patient):
-			_log.error('cannot load item links for patient [%s]' % self.pk_patient)
-			curs.close()
-			return None
-		rows = curs.fetchall()
-		view_col_idx = gmPG2.get_col_indices(curs)
-
-		# aggregate by src_table for item retrieval
-		items_by_table = {}
-		for item in rows:
-			src_table = item[view_col_idx['src_table']]
-			pk_item = item[view_col_idx['pk_item']]
-			if not items_by_table.has_key(src_table):
-				items_by_table[src_table] = {}
-			items_by_table[src_table][pk_item] = item
-
-		# get mapping for issue/episode IDs
-		issues = self.get_health_issues()
-		issue_map = {}
-		for issue in issues:
-			issue_map[issue['pk']] = issue['description']
-		episodes = self.get_episodes()
-		episode_map = {}
-		for episode in episodes:
-			episode_map[episode['pk_episode']] = episode['description']
-		emr_data = {}
-		# get item data from all source tables
-		for src_table in items_by_table.keys():
-			item_ids = items_by_table[src_table].keys()
-			# we don't know anything about the columns of
-			# the source tables but, hey, this is a dump
-			if len(item_ids) == 0:
-				_log.info('no items in table [%s] ?!?' % src_table)
-				continue
-			elif len(item_ids) == 1:
-				cmd = "SELECT * FROM %s WHERE pk_item=%%s order by modified_when" % src_table
-				if not gmPG2.run_query(curs, None, cmd, item_ids[0]):
-					_log.error('cannot load items from table [%s]' % src_table)
-					# skip this table
-					continue
-			elif len(item_ids) > 1:
-				cmd = "SELECT * FROM %s WHERE pk_item in %%s order by modified_when" % src_table
-				if not gmPG.run_query(curs, None, cmd, (tuple(item_ids),)):
-					_log.error('cannot load items from table [%s]' % src_table)
-					# skip this table
-					continue
-			rows = curs.fetchall()
-			table_col_idx = gmPG.get_col_indices(curs)
-			# format per-table items
-			for row in rows:
-				# FIXME: make this get_pkey_name()
-				pk_item = row[table_col_idx['pk_item']]
-				view_row = items_by_table[src_table][pk_item]
-				age = view_row[view_col_idx['age']]
-				# format metadata
-				try:
-					episode_name = episode_map[view_row[view_col_idx['pk_episode']]]
-				except:
-					episode_name = view_row[view_col_idx['pk_episode']]
-				try:
-					issue_name = issue_map[view_row[view_col_idx['pk_health_issue']]]
-				except:
-					issue_name = view_row[view_col_idx['pk_health_issue']]
-
-				if not emr_data.has_key(age):
-					emr_data[age] = []
-
-				emr_data[age].append(
-					_('%s: encounter (%s)') % (
-						view_row[view_col_idx['clin_when']],
-						view_row[view_col_idx['pk_encounter']]
-					)
-				)
-				emr_data[age].append(_('health issue: %s') % issue_name)
-				emr_data[age].append(_('episode     : %s') % episode_name)
-				# format table specific data columns
-				# - ignore those, they are metadata, some
-				#   are in clin.v_pat_items data already
-				cols2ignore = [
-					'pk_audit', 'row_version', 'modified_when', 'modified_by',
-					'pk_item', 'id', 'fk_encounter', 'fk_episode'
-				]
-				col_data = []
-				for col_name in table_col_idx.keys():
-					if col_name in cols2ignore:
-						continue
-					emr_data[age].append("=> %s:" % col_name)
-					emr_data[age].append(row[table_col_idx[col_name]])
-				emr_data[age].append("----------------------------------------------------")
-				emr_data[age].append("-- %s from table %s" % (
-					view_row[view_col_idx['modified_string']],
-					src_table
-				))
-				emr_data[age].append("-- written %s by %s" % (
-					view_row[view_col_idx['modified_when']],
-					view_row[view_col_idx['modified_by']]
-				))
-				emr_data[age].append("----------------------------------------------------")
-		curs.close()
-		self._conn_pool.ReleaseConnection('historica')
-		return emr_data
-	#--------------------------------------------------------
 	def get_text_dump(self, since=None, until=None, encounters=None, episodes=None, issues=None):
-		# don't know how to invalidate this by means of
-		# a notify without catching notifies from *all*
-		# child tables, the best solution would be if
-		# inserts in child tables would also fire triggers
-		# of ancestor tables, but oh well,
-		# until then the text dump will not be cached ...
-		try:
-			return self.__db_cache['text dump']
-		except KeyError:
-			pass
-		# not cached so go get it
-		# -- get the data --
 		fields = [
 			'age',
 			"to_char(modified_when, 'YYYY-MM-DD @ HH24:MI') as modified_when",
@@ -766,16 +575,18 @@ order by
 			u'SELECT count(1) FROM clin.v_pat_items WHERE pk_patient = %(pat)s',
 			u'SELECT count(1) FROM blobs.v_doc_med WHERE pk_patient = %(pat)s',
 			u'SELECT count(1) FROM clin.v_test_results WHERE pk_patient = %(pat)s',
-			u'SELECT count(1) FROM clin.v_pat_hospital_stays WHERE pk_patient = %(pat)s',
-			u'SELECT count(1) FROM clin.v_pat_procedures WHERE pk_patient = %(pat)s',
+			u'SELECT count(1) FROM clin.v_hospital_stays WHERE pk_patient = %(pat)s',
+			u'SELECT count(1) FROM clin.v_procedures WHERE pk_patient = %(pat)s',
 			# active and approved substances == medication
 			u"""
 				SELECT count(1)
-				from clin.v_pat_substance_intake
+				FROM clin.v_substance_intakes
 				WHERE
 					pk_patient = %(pat)s
-					and is_currently_active in (null, true)
-					and intake_is_approved_of in (null, true)""",
+						AND
+					is_currently_active IN (null, true)
+						AND
+					intake_is_approved_of IN (null, true)""",
 			u'SELECT count(1) FROM clin.v_pat_vaccinations WHERE pk_patient = %(pat)s'
 		])
 
@@ -811,7 +622,12 @@ order by
 			'Vaccinations: %(vaccinations)s'
 		) % self.get_statistics()
 	#--------------------------------------------------------
-	def format_summary(self, dob=None):
+	def format_summary(self):
+
+		cmd = u"SELECT dob from dem.v_basic_person where pk_identity = %(pk)s"
+		args = {'pk': self.pk_patient}
+		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = False)
+		dob = rows[0]['dob']
 
 		stats = self.get_statistics()
 		first = self.get_first_encounter()
@@ -866,27 +682,38 @@ order by
 				gmTools.coalesce(allg['reaction'], _('unknown reaction'))
 			)
 
-		txt += u'\n'
-		txt += _('Family History')
-		txt += u'\n'
+		meds = self.get_current_substance_intakes(order_by = u'intake_is_approved_of DESC, substance')
+		if len(meds) > 0:
+			txt += u'\n'
+			txt += _('Medications and Substances')
+			txt += u'\n'
+		for m in meds:
+			txt += u'%s\n' % m.format_as_one_line(left_margin = 1)
+
 		fhx = self.get_family_history()
+		if len(fhx) > 0:
+			txt += u'\n'
+			txt += _('Family History')
+			txt += u'\n'
 		for f in fhx:
 			txt += u'%s\n' % f.format(left_margin = 1)
 
-		txt += u'\n'
-		txt += _('Occupations')
-		txt += u'\n'
 		jobs = get_occupations(pk_identity = self.pk_patient)
+		if len(jobs) > 0:
+			txt += u'\n'
+			txt += _('Occupations')
+			txt += u'\n'
 		for job in jobs:
 			txt += u' %s%s\n' % (
 				job['l10n_occupation'],
 				gmTools.coalesce(job['activities'], u'', u': %s')
 			)
 
-		txt += u'\n'
-		txt += _('Vaccinations')
-		txt += u'\n'
 		vaccs = self.get_latest_vaccinations()
+		if len(vaccs) > 0:
+			txt += u'\n'
+			txt += _('Vaccinations')
+			txt += u'\n'
 		inds = sorted(vaccs.keys())
 		for ind in inds:
 			ind_count, vacc = vaccs[ind]
@@ -1326,28 +1153,27 @@ WHERE
 	#--------------------------------------------------------
 	# API: substance intake
 	#--------------------------------------------------------
-	def get_current_substance_intake(self, include_inactive=True, include_unapproved=False, order_by=None, episodes=None, issues=None):
+	def get_current_substance_intakes(self, include_inactive=True, include_unapproved=False, order_by=None, episodes=None, issues=None):
 
 		where_parts = [u'pk_patient = %(pat)s']
+		args = {'pat': self.pk_patient}
 
 		if not include_inactive:
-			where_parts.append(u'is_currently_active in (true, null)')
+			where_parts.append(u'is_currently_active IN (true, null)')
 
 		if not include_unapproved:
-			where_parts.append(u'intake_is_approved_of in (true, null)')
+			where_parts.append(u'intake_is_approved_of IN (true, null)')
 
 		if order_by is None:
 			order_by = u''
 		else:
-			order_by = u'order by %s' % order_by
+			order_by = u'ORDER BY %s' % order_by
 
-		cmd = u"SELECT * FROM clin.v_pat_substance_intake WHERE %s %s" % (
-			u'\nand '.join(where_parts),
+		cmd = u"SELECT * FROM clin.v_substance_intakes WHERE %s %s" % (
+			u'\nAND '.join(where_parts),
 			order_by
 		)
-
-		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': {'pat': self.pk_patient}}], get_col_idx = True)
-
+		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = True)
 		meds = [ gmMedication.cSubstanceIntakeEntry(row = {'idx': idx, 'data': r, 'pk_field': 'pk_substance_intake'})  for r in rows ]
 
 		if episodes is not None:
@@ -1473,22 +1299,17 @@ WHERE
 				regimes for, must be primary language, not l10n_indication
 		"""
 		# FIXME: use course, not regime
-		try:
-			self.__db_cache['vaccinations']['scheduled regimes']
-		except KeyError:
-			# retrieve vaccination regimes definitions
-			self.__db_cache['vaccinations']['scheduled regimes'] = []
-			cmd = """SELECT distinct on(pk_course) pk_course
-					 FROM clin.v_vaccs_scheduled4pat
-					 WHERE pk_patient=%s"""
-			rows = gmPG.run_ro_query('historica', cmd, None, self.pk_patient)
-			if rows is None:
-				_log.error('cannot retrieve scheduled vaccination courses')
-				del self.__db_cache['vaccinations']['scheduled regimes']
-				return None
-			# Instantiate vaccination items and keep cache
-			for row in rows:
-				self.__db_cache['vaccinations']['scheduled regimes'].append(gmVaccination.cVaccinationCourse(aPK_obj=row[0]))
+		# retrieve vaccination regimes definitions
+		cmd = """SELECT distinct on(pk_course) pk_course
+				 FROM clin.v_vaccs_scheduled4pat
+				 WHERE pk_patient=%s"""
+		rows = gmPG.run_ro_query('historica', cmd, None, self.pk_patient)
+		if rows is None:
+			_log.error('cannot retrieve scheduled vaccination courses')
+			return None
+		# Instantiate vaccination items and keep cache
+		for row in rows:
+			self.__db_cache['vaccinations']['scheduled regimes'].append(gmVaccination.cVaccinationCourse(aPK_obj=row[0]))
 
 		# ok, let's constrain our list
 		filtered_regimes = []
@@ -1546,7 +1367,7 @@ WHERE
 		"""
 		try:
 			self.__db_cache['vaccinations']['vaccinated']
-		except KeyError:			
+		except KeyError:
 			self.__db_cache['vaccinations']['vaccinated'] = []
 			# Important fetch ordering by indication, date to know if a vaccination is booster
 			cmd= """SELECT * FROM clin.v_pat_vaccinations4indication
@@ -1717,13 +1538,21 @@ WHERE
 				_log.debug('unsaved changes in active encounter, cannot switch to another one')
 				raise ValueError('unsaved changes in active encounter, cannot switch to another one')
 
-		# set the currently active encounter and announce that change
-		if encounter['started'].strftime('%Y-%m-%d %H:%M') == encounter['last_affirmed'].strftime('%Y-%m-%d %H:%M'):
-			now = gmDateTime.pydt_now_here()
-			if now > encounter['started']:
-				encounter['last_affirmed'] = now		# this will trigger an "encounter_mod_db"
-				encounter.save()
+		# be more conservative, it seems to have brought about
+		# races involving encounter mod signals which made GNUmed crash
+#		# set the currently active encounter and announce that change
+#		if encounter['started'].strftime('%Y-%m-%d %H:%M') == encounter['last_affirmed'].strftime('%Y-%m-%d %H:%M'):
+#			now = gmDateTime.pydt_now_here()
+#			if now > encounter['started']:
+#				encounter['last_affirmed'] = now		# this will trigger an "clin.encounter_mod_db"
+#				encounter.save()
+		prev_enc = None
+		if self.__encounter is not None:
+			prev_enc = self.__encounter
+		encounter.lock(exclusive = False)		# lock new
 		self.__encounter = encounter
+		if prev_enc is not None:				# unlock old
+			prev_enc.unlock(exclusive = False)
 		gmDispatcher.send(u'current_encounter_switched')
 
 		return True
@@ -1845,8 +1674,10 @@ WHERE
 			'\n'
 			' %s  %s - %s  (%s)\n'
 			'\n'
-			' Request: %s\n'
-			' Outcome: %s\n'
+			' Reason for Encounter:\n'
+			'  %s\n'
+			' Assessment of Encounter:\n'
+			'  %s\n'
 			'\n'
 			'Do you want to continue that consultation\n'
 			'or do you want to start a new one ?\n'
@@ -1874,14 +1705,19 @@ WHERE
 	#------------------------------------------------------------------
 	def start_new_encounter(self):
 		cfg_db = gmCfg.cCfgSQL()
-		# FIXME: look for MRU/MCU encounter type config here
 		enc_type = cfg_db.get2 (
 			option = u'encounter.default_type',
 			workplace = _here.active_workplace,
-			bias = u'user',
-			default = u'in surgery'
+			bias = u'user'
 		)
-		self.current_encounter = gmEMRStructItems.create_encounter(fk_patient = self.pk_patient, enc_type = enc_type)
+		if enc_type is None:
+			enc_type = gmEMRStructItems.get_most_commonly_used_encounter_type()
+		if enc_type is None:
+			enc_type = u'in surgery'
+		enc = gmEMRStructItems.create_encounter(fk_patient = self.pk_patient, enc_type = enc_type)
+		enc['pk_org_unit'] = _here['pk_org_unit']
+		enc.save()
+		self.current_encounter = enc
 		_log.debug('new encounter [%s] initiated' % self.current_encounter['pk_encounter'])
 	#------------------------------------------------------------------
 	def get_encounters(self, since=None, until=None, id_list=None, episodes=None, issues=None, skip_empty=False):
@@ -2078,13 +1914,12 @@ SELECT MIN(earliest) FROM (
 		args = {'pat': self.pk_patient}
 
 		if (issue_id is None) and (episode_id is None):
-
 			cmd = u"""
-SELECT * FROM clin.v_pat_encounters
-WHERE pk_patient = %(pat)s
-ORDER BY started DESC
-LIMIT 2
-"""
+				SELECT * FROM clin.v_pat_encounters
+				WHERE pk_patient = %(pat)s
+				ORDER BY started DESC
+				LIMIT 2
+			"""
 		else:
 			where_parts = []
 
@@ -2097,20 +1932,20 @@ LIMIT 2
 				args['epi'] = episode_id
 
 			cmd = u"""
-SELECT *
-FROM clin.v_pat_encounters
-WHERE
-	pk_patient = %%(pat)s
-		AND
-	pk_encounter IN (
-		SELECT distinct pk_encounter
-		FROM clin.v_pat_narrative
-		WHERE
-			%s
-	)
-ORDER BY started DESC
-LIMIT 2
-""" % u' AND '.join(where_parts)
+				SELECT *
+				FROM clin.v_pat_encounters
+				WHERE
+					pk_patient = %%(pat)s
+						AND
+					pk_encounter IN (
+						SELECT distinct pk_encounter
+						FROM clin.v_narrative
+						WHERE
+							%s
+					)
+				ORDER BY started DESC
+				LIMIT 2
+			""" % u' AND '.join(where_parts)
 
 		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = True)
 
@@ -2142,7 +1977,7 @@ LIMIT 2
 		)
 
 #		# FIXME: this should be done async
-		cmd = u"select clin.remove_old_empty_encounters(%(pat)s::integer, %(ttl)s::interval)"
+		cmd = u"SELECT clin.remove_old_empty_encounters(%(pat)s::INTEGER, %(ttl)s::INTERVAL)"
 		args = {'pat': self.pk_patient, 'ttl': ttl}
 		try:
 			rows, idx = gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}])
@@ -2275,7 +2110,7 @@ LIMIT 2
 			type = type['pk_test_type']
 
 		if intended_reviewer is None:
-			intended_reviewer = _me['pk_staff']
+			intended_reviewer = gmStaff.gmCurrentProvider()['pk_staff']
 
 		tr = gmPathLab.create_test_result (
 			encounter = self.current_encounter['pk_encounter'],
@@ -2418,7 +2253,7 @@ if __name__ == "__main__":
 	#-----------------------------------------
 	def test_get_meds():
 		emr = cClinicalRecord(aPKey=12)
-		for med in emr.get_current_substance_intake():
+		for med in emr.get_current_substance_intakes():
 			print med
 	#-----------------------------------------
 	def test_is_allergic_to():
